@@ -45,7 +45,6 @@ struct SoupSSLCredentials {
 
 typedef struct {
 	GIOChannel channel;
-	int fd;
 	GIOChannel *real_sock;
 	gboolean non_blocking;
 	gnutls_session session;
@@ -207,6 +206,19 @@ again:
 			goto again;
 	}
 
+	if (result == GNUTLS_E_UNEXPECTED_PACKET_LENGTH) {
+		/* This means the connection was either corrupted or
+		 * interrupted. One particular thing that it can mean
+		 * is that the remote end closed the connection
+		 * abruptly without doing a proper TLS Close. There
+		 * are security reasons why it's bad to treat this as
+		 * not-an-error, but for compatibility reasons (eg,
+		 * bug 577386) we kinda have to. And it's not like
+		 * we're very secure anyway.
+		 */
+		return G_IO_STATUS_EOF;
+	}
+
 	if (result < 0) {
 		g_set_error (err, G_IO_CHANNEL_ERROR,
 			     G_IO_CHANNEL_ERROR_FAILED,
@@ -279,7 +291,7 @@ soup_gnutls_seek (GIOChannel  *channel,
 {
 	SoupGNUTLSChannel *chan = (SoupGNUTLSChannel *) channel;
 
-	return chan->real_sock->funcs->io_seek (channel, offset, type, err);
+	return chan->real_sock->funcs->io_seek (chan->real_sock, offset, type, err);
 }
 
 static GIOStatus
@@ -296,7 +308,7 @@ soup_gnutls_close (GIOChannel  *channel,
 		} while (ret == GNUTLS_E_INTERRUPTED);
 	}
 
-	return chan->real_sock->funcs->io_close (channel, err);
+	return chan->real_sock->funcs->io_close (chan->real_sock, err);
 }
 
 static GSource *
@@ -305,7 +317,7 @@ soup_gnutls_create_watch (GIOChannel   *channel,
 {
 	SoupGNUTLSChannel *chan = (SoupGNUTLSChannel *) channel;
 
-	return chan->real_sock->funcs->io_create_watch (channel,
+	return chan->real_sock->funcs->io_create_watch (chan->real_sock,
 							condition);
 }
 
@@ -326,7 +338,7 @@ soup_gnutls_set_flags (GIOChannel  *channel,
 {
 	SoupGNUTLSChannel *chan = (SoupGNUTLSChannel *) channel;
 
-	return chan->real_sock->funcs->io_set_flags (channel, flags, err);
+	return chan->real_sock->funcs->io_set_flags (chan->real_sock, flags, err);
 }
 
 static GIOFlags
@@ -334,7 +346,7 @@ soup_gnutls_get_flags (GIOChannel *channel)
 {
 	SoupGNUTLSChannel *chan = (SoupGNUTLSChannel *) channel;
 
-	return chan->real_sock->funcs->io_get_flags (channel);
+	return chan->real_sock->funcs->io_get_flags (chan->real_sock);
 }
 
 static const GIOFuncs soup_gnutls_channel_funcs = {
@@ -421,7 +433,6 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, gboolean non_blocking,
 	gnutls_transport_set_ptr (session, GINT_TO_POINTER (sockfd));
 
 	chan = g_slice_new0 (SoupGNUTLSChannel);
-	chan->fd = sockfd;
 	chan->real_sock = sock;
 	chan->session = session;
 	chan->creds = creds;
